@@ -1,5 +1,8 @@
 #' Available Sensors
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' A list containing all available sensors in this package you can work with. This variable was
 #' created so it is easier to use in your own functions, e.g. to loop over sensors.
 #'
@@ -7,13 +10,18 @@
 #' @examples
 #' sensors
 #' @export sensors
-sensors <- c("Accelerometer", "AirQuality", "Activity", "AppUsage", "Battery", "Bluetooth",
-             "Calendar", "Connectivity", "Device", "Error", "Geofence", "Gyroscope",
-             "InstalledApps", "Keyboard", "Light", "Location", "Memory", "Mobility", "Noise",
-             "Pedometer", "PhoneLog", "Screen", "TextMessage", "Weather", "Wifi")
+sensors <- c(
+  "Accelerometer", "AirQuality", "Activity", "AppUsage", "Battery", "Bluetooth",
+  "Calendar", "Connectivity", "Device", "Error", "Geofence", "Gyroscope",
+  "InstalledApps", "Keyboard", "Light", "Location", "Memory", "Mobility", "Noise",
+  "Pedometer", "PhoneLog", "Screen", "TextMessage", "Weather", "Wifi"
+)
 
 #' Create a new mpathsenser database
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @param path The path to the database.
 #' @param db_name The name of the database.
 #' @param overwrite In case a database with `db_name` already exists, indicate whether it should
@@ -34,8 +42,9 @@ create_db <- function(path = getwd(), db_name = "sense.db", overwrite = FALSE) {
   if (file.exists(db_name)) {
     if (overwrite) {
       tryCatch(file.remove(db_name),
-               warning = function(e) stop(warningCondition(e)),
-               error = function(e) stop(errorCondition(e)))
+        warning = function(e) stop(warningCondition(e)),
+        error = function(e) stop(errorCondition(e))
+      )
     } else {
       stop(paste("Database", db_name, "already exists. Use overwrite = TRUE to overwrite."))
     }
@@ -45,22 +54,28 @@ create_db <- function(path = getwd(), db_name = "sense.db", overwrite = FALSE) {
   db <- DBI::dbConnect(RSQLite::SQLite(), db_name, cache_size = 8192)
 
   # Populate the db with empty tables
-  tryCatch({
-    fn <- system.file("extdata", "dbdef.sql", package = "mpathsenser")
-    script <- strsplit(paste0(readLines(fn, warn = FALSE), collapse = "\n"),  "\n\n")[[1]]
-    for (statement in script) {
-      DBI::dbExecute(db, statement)
+  tryCatch(
+    {
+      fn <- system.file("extdata", "dbdef.sql", package = "mpathsenser")
+      script <- strsplit(paste0(readLines(fn, warn = FALSE), collapse = "\n"), "\n\n")[[1]]
+      for (statement in script) {
+        DBI::dbExecute(db, statement)
+      }
+    },
+    error = function(e) {
+      DBI::dbDisconnect(db)
+      stop(e)
     }
-  }, error = function(e) {
-    DBI::dbDisconnect(db)
-    stop(e)
-  })
+  )
 
   return(db)
 }
 
 #' Open an mpathsenser database.
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @param path The path to the database. Use NULL to use the full path name in db_name.
 #' @param db_name The name of the database.
 #'
@@ -80,8 +95,9 @@ open_db <- function(path = getwd(), db_name = "sense.db") {
     db_name <- suppressWarnings(normalizePath(file.path(path, db_name)))
   }
 
-  if (!file.exists(db_name))
+  if (!file.exists(db_name)) {
     stop("There is no such file")
+  }
   db <- DBI::dbConnect(RSQLite::SQLite(), db_name, cache_size = 8192)
   if (!DBI::dbExistsTable(db, "Participant")) {
     DBI::dbDisconnect(db)
@@ -92,6 +108,9 @@ open_db <- function(path = getwd(), db_name = "sense.db") {
 
 #' Close a database connection
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' This is a convenience function that is simply a wrapper around \link[DBI]{dbDisconnect}.
 #'
 #' @inheritParams get_data
@@ -112,6 +131,9 @@ close_db <- function(db) {
 
 #' Create indexes for a mpathsenser database
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @inheritParams get_data
 #'
 #' @return No return value, called for side effects.
@@ -119,15 +141,18 @@ close_db <- function(db) {
 index_db <- function(db) {
   if (is.null(db) || !DBI::dbIsValid(db)) stop("Database connection is not valid")
 
-  tryCatch({
-    fn <- system.file("extdata", "indexes.sql", package = "mpathsenser")
-    script <- strsplit(paste0(readLines(fn, warn = FALSE), collapse = "\n"),  "\n\n")[[1]]
-    for (statement in script) {
-      DBI::dbExecute(db, statement)
+  tryCatch(
+    {
+      fn <- system.file("extdata", "indexes.sql", package = "mpathsenser")
+      script <- strsplit(paste0(readLines(fn, warn = FALSE), collapse = "\n"), "\n\n")[[1]]
+      for (statement in script) {
+        DBI::dbExecute(db, statement)
+      }
+    },
+    error = function(e) {
+      stop(e)
     }
-  }, error = function(e) {
-    stop(e)
-  })
+  )
 }
 
 vacuum_db <- function(db) {
@@ -137,6 +162,9 @@ vacuum_db <- function(db) {
 
 #' Copy (a subset of) a database to another database
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @param from_db A mpathsenser database connection from where the data will be transferred.
 #' @param to_db A mpathsenser database connection where the data will be transferred to. If no
 #' new_db is specified, a path (and possibly a db_name) must be specified for
@@ -168,10 +196,12 @@ copy_db <- function(from_db, to_db = NULL, sensor = "All", path = getwd(), db_na
 
     if (!file.exists(file.path(path, db_name))) {
       to_db <- create_db(path, db_name)
-      message(paste0("New database created in ", path))
+      message(paste0("New database created in ", file.path(path, db_name)))
     } else {
-      stop(paste0("A file in ", path, " with the name ", db_name, " already exists. Please choose ",
-                  "a different name or path or remove the file."))
+      stop(paste0(
+        "A file in ", path, " with the name ", db_name, " already exists. Please choose ",
+        "a different name or path or remove the file."
+      ))
     }
   }
 
@@ -181,14 +211,18 @@ copy_db <- function(from_db, to_db = NULL, sensor = "All", path = getwd(), db_na
   # Copy participants, studies, processed_files
   DBI::dbExecute(from_db, "INSERT OR IGNORE INTO new_db.Study SELECT * FROM Study")
   DBI::dbExecute(from_db, "INSERT OR IGNORE INTO new_db.Participant SELECT * FROM Participant")
-  DBI::dbExecute(from_db,
-                 "INSERT OR IGNORE INTO new_db.ProcessedFiles SELECT * FROM ProcessedFiles")
+  DBI::dbExecute(
+    from_db,
+    "INSERT OR IGNORE INTO new_db.ProcessedFiles SELECT * FROM ProcessedFiles"
+  )
 
 
   # Copy all specified sensors
   for (i in seq_along(sensor)) {
-    DBI::dbExecute(from_db, paste0("INSERT OR IGNORE INTO new_db.", sensor[i],
-                                   " SELECT * FROM ", sensor[i]))
+    DBI::dbExecute(from_db, paste0(
+      "INSERT OR IGNORE INTO new_db.", sensor[i],
+      " SELECT * FROM ", sensor[i]
+    ))
   }
 
   # Detach
@@ -202,26 +236,34 @@ copy_db <- function(from_db, to_db = NULL, sensor = "All", path = getwd(), db_na
 }
 
 add_study <- function(db, data) {
-  DBI::dbExecute(db,
-                 "INSERT OR IGNORE INTO Study(study_id, data_format)
+  DBI::dbExecute(
+    db,
+    "INSERT OR IGNORE INTO Study(study_id, data_format)
   VALUES(:study_id, :data_format);",
-                 list(study_id = data$study_id, data_format = data$data_format))
+    list(study_id = data$study_id, data_format = data$data_format)
+  )
 }
 
 add_participant <- function(db, data) {
-  DBI::dbExecute(db,
-                 "INSERT OR IGNORE INTO Participant(participant_id, study_id)
+  DBI::dbExecute(
+    db,
+    "INSERT OR IGNORE INTO Participant(participant_id, study_id)
   VALUES(:participant_id, :study_id);",
-                 list(participant_id = data$participant_id, study_id = data$study_id))
+    list(participant_id = data$participant_id, study_id = data$study_id)
+  )
 }
 
 add_processed_files <- function(db, data) {
-  DBI::dbExecute(db,
-                 "INSERT OR IGNORE INTO ProcessedFiles(file_name, study_id, participant_id)
+  DBI::dbExecute(
+    db,
+    "INSERT OR IGNORE INTO ProcessedFiles(file_name, study_id, participant_id)
   VALUES(:file_name, :study_id, :participant_id);",
-                 list(file_name = data$file_name,
-                      study_id = data$study_id,
-                      participant_id = data$participant_id))
+    list(
+      file_name = data$file_name,
+      study_id = data$study_id,
+      participant_id = data$participant_id
+    )
+  )
 }
 
 clear_sensors_db <- function(db) {
@@ -235,6 +277,9 @@ clear_sensors_db <- function(db) {
 
 #' Get all processed files from a database
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @param db A database connection, as created by \link[mpathsenser]{create_db}.
 #'
 #' @return A data frame containing the \code{file_name}, \code{participant_id}, and \code{study_id}
@@ -247,6 +292,9 @@ get_processed_files <- function(db) {
 
 #' Get all participants
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @param db db A database connection, as created by \link[mpathsenser]{create_db}.
 #' @param lazy Whether to evaluate lazily using \link[dbplyr]{dbplyr}.
 #'
@@ -263,6 +311,9 @@ get_participants <- function(db, lazy = FALSE) {
 
 #' Get all studies
 #'
+#' @description
+#' `r lifecycle::badge("stable")`
+#'
 #' @param db db A database connection, as created by \link[mpathsenser]{create_db}.
 #' @param lazy Whether to evaluate lazily using \link[dbplyr]{dbplyr}.
 #'
@@ -277,7 +328,10 @@ get_studies <- function(db, lazy = FALSE) {
   }
 }
 
-#' Get the number of rows sensors in a mpathsenser database
+#' Get the number of rows per sensor in a mpathsenser database
+#'
+#' @description
+#' `r lifecycle::badge("stable")`
 #'
 #' @param db db A database connection, as created by \link[mpathsenser]{create_db}.
 #' @param sensor A character vector of one or multiple vectors. Use "All" for all sensors. See
